import argparse
from blueness import module
from openai_commands import NAME, VERSION
from openai_commands import env
from openai_commands.gpt.chat import chat_with_openai, list_models
from openai_commands.logger import logger
from blueness.argparse.generic import sys_exit

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME, description=f"{NAME}-{VERSION}")
parser.add_argument(
    "task",
    type=str,
    help="chat_with_openai|list_models",
)
parser.add_argument(
    "--object_path",
    type=str,
    default="",
)
parser.add_argument(
    "--model_name",
    type=str,
    default=env.OPENAI_GPT_DEFAULT_MODEL,
    help='"gpt list_models".',
)
parser.add_argument(
    "--log",
    type=int,
    default=1,
    help="0|1",
)
args = parser.parse_args()

success = False
if args.task == "chat_with_openai":
    success, _ = chat_with_openai(
        output_path=args.object_path,
        model_name=args.model_name,
    )
elif args.task == "list_models":
    success = True
    list_models(log=bool(args.log))
else:
    success = None

sys_exit(logger, NAME, args.task, success)



import glob
import pandas as pd

from blueness import module
from blue_objects import file, objects

from openai_commands import NAME
from openai_commands.logger import logger

NAME = module.name(__file__, NAME)


def combine(
    object_name_1: str,
    object_name_2: str,
    object_name: str,
    log: bool = True,
) -> bool:
    logger.info(
        "{}.combine: {} + {} -> {}".format(
            NAME,
            object_name_1,
            object_name_2,
            object_name,
        )
    )

    for filename_1 in glob.glob(objects.path_of("*.csv", object_name_1)):
        filename_name_and_extension = file.name_and_extension(filename_1)

        filename_2 = objects.path_of(
            filename_name_and_extension,
            object_name_2,
        )

        if not file.exists(filename_2):
            logger.warning(
                "missing in object-2 [{}]: {}".format(
                    object_name_2,
                    filename_name_and_extension,
                )
            )
            continue

        success_1, df_1 = file.load_dataframe(filename_1, log=log)
        success_2, df_2 = file.load_dataframe(filename_2, log=log)
        if not success_1 or not success_2:
            logger.warning("bad file: {}".format(filename_name_and_extension))
            continue

        df = pd.concat([df_1, df_2], axis=1)
        df = df.loc[:, ~df.columns.duplicated()]

        if not file.save_csv(
            objects.path_of(
                filename_name_and_extension,
                object_name,
            ),
            df,
            log=log,
        ):
            return False

    return True



from typing import Dict, Any, Tuple

from blue_options.logger import crash_report

from openai_commands.completion.functions.generic import ai_function
from openai_commands.logger import logger


class ai_function_py(ai_function):
    def __init__(
        self,
        output_class_name: str = "",
        verbose=None,
        validation_input=None,
    ):
        super().__init__(verbose=verbose)

        self.language = "python"
        self.output_class_name = output_class_name
        self.validation_input = validation_input

        self.function_handle = None

    def compute(self, inputs):
        if self.function_handle is None:
            return None

        logger.info(
            "{}.compute({})".format(
                self.__class__.__name__,
                inputs.__class__.__name__,
            )
        )

        outputs = self.function_handle(inputs)
        logger.info("-> {}".format(outputs.__class__.__name__))

        if self.output_class_name:
            if outputs.__class__.__name__ != self.output_class_name:
                logger.error(
                    "{}.compute({}): {} != {}".format(
                        self.__class__.__name__,
                        inputs.__class__.__name__,
                        outputs.__class__.__name__,
                        self.output_class_name,
                    )
                )
                return None

        return outputs

    def generate(
        self,
        prompt,
        retry: int = 5,
    ) -> Tuple[bool, Dict[str, Any]]:
        success, metadata = super().generate(prompt)
        if not success:
            return success, metadata

        self.function_handle = None

        try:
            exec(self.code)
        except:
            crash_report("{}.generate() failed.".format(self.__class__.__name__))

        for thing in locals().values():
            if hasattr(thing, "__name__") and thing.__name__ == self.function_name:
                self.function_handle = thing
                break

        passed = self.function_handle is not None

        if passed and self.validation_input is not None:
            try:
                validation_output = self.compute(self.validation_input)

                if validation_output is None:
                    passed = False
            except:
                crash_report("compute test failed.")
                passed = False

        if passed:
            if self.auto_save:
                self.save()
            return True, metadata

        if retry <= 0:
            logger.info("last retry, failed.")
            return False, metadata

        logger.info("{} more tries.".format(retry))
        return self.generate(prompt, retry - 1)

    def to_json(self):
        output = super().to_json()
        output.update({"output_class_name": self.output_class_name})
        return output



"""
This Python script enables real-time conversation with OpenAI's ChatGPT model through the terminal.
Generated by ChatGPT, this script establishes a session where the user can interactively ask questions
or have a dialogue with the ChatGPT model. The user's input is sent to the ChatGPT API, and the model's
responses are displayed in the terminal, facilitating a conversational interface. Ensure you have the `openai`
Python package installed and an OpenAI API key set before running this script.

committed in its original form and then modified by Arash Abadpour - arash.abadpour@gmail.com.
"""

import os
from typing import List, Any, Tuple
from openai import OpenAI

from blueness import module
from blue_objects import file, path

from openai_commands import NAME, VERSION
from openai_commands import env
from openai_commands.logger import logger

NAME = module.name(__file__, NAME)

FULL_NAME = f"{NAME}-{VERSION}"

client = OpenAI(api_key=env.OPENAI_API_KEY)


def chat_with_openai(
    output_path: str = "",
    script_mode: bool = False,
    script: List[str] = [],
    model_name: str = env.OPENAI_GPT_DEFAULT_MODEL,
) -> Tuple[bool, List[Any]]:
    logger.info(
        "{} @ {}{}".format(
            FULL_NAME,
            model_name,
            f": 📜 {len(script)} line(s)." if script_mode else "",
        )
    )
    logger.info("ChatGPT: Hello! How can I assist you today?")

    conversation: List[Any] = []
    index = -1
    while True:
        index += 1

        if script_mode:
            if index >= len(script):
                logger.info("end of script.")
                break

            logger.info(
                "script: line #{}/{}: {}".format(
                    index + 1,
                    len(script),
                    script[index],
                )
            )

        user_input = script[index] if script_mode else input("(?:help) > ")

        if user_input in ["?", "help"]:
            logger.info("exit: exit.")
            logger.info("version: show version.")
            continue
        if user_input == "version":
            logger.info(FULL_NAME)
            continue
        if user_input == "exit":
            break

        response = client.chat.completions.create(
            model=model_name,
            messages=[{"role": "user", "content": user_input}],
            max_tokens=150,
        )

        answer = response.choices[0].message.content.strip()
        logger.info(f"ChatGPT: {answer}")

        conversation.append(
            {
                "user_input": user_input,
                "answer": answer,
            }
        )

    return (
        not (output_path)
        or file.save_yaml(
            os.path.join(
                output_path,
                f"{path.name(output_path)}.yaml",
            ),
            {
                "conversation": conversation,
                "created-by": FULL_NAME,
                "script_mode": script_mode,
            },
        ),
        conversation,
    )


def interact_with_openai(
    prompt: str,
    output_path: str = "",
    model_name: str = env.OPENAI_GPT_DEFAULT_MODEL,
) -> Tuple[bool, List[str]]:
    success, conversation = chat_with_openai(
        output_path=output_path,
        script_mode=True,
        script=[prompt],
        model_name=model_name,
    )

    if not conversation:
        return False, ""

    return success, conversation[0].get("answer", "").split("\n")


def list_models(log: bool = False) -> List[Any]:
    list_of_models = client.models.list().data

    if log:
        logger.info(f"{len(list_of_models)} model(s)")
        for index, model in enumerate(list_of_models):
            logger.info(f" #{index}: {model}")

    return list_of_models



import argparse
import base64
from blueness import module
from openai_commands import NAME, VERSION
from openai_commands.literature_review.combination import combine
from openai_commands.literature_review.functions import review_literature
from openai_commands.literature_review.multiple import (
    generate_workflow as generate_multiple_review_workflow,
)
from openai_commands.logger import logger
from blueness.argparse.generic import sys_exit

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME, description=f"{NAME}-{VERSION}")
parser.add_argument(
    "task",
    type=str,
    default="",
    help="combine|generate_multiple_review_workflow|review",
)
parser.add_argument(
    "--args",
    type=str,
)
parser.add_argument(
    "--question",
    type=str,
    default="question1",
    help="<question>",
)
parser.add_argument(
    "--count",
    type=int,
    default=10,
    help="-1: all",
)
parser.add_argument(
    "--do_publish",
    type=int,
    default=0,
    help="0|1",
)
parser.add_argument(
    "--filename",
    type=str,
    default="",
    help="<review.csv>",
)
parser.add_argument(
    "--input_object_name",
    type=str,
)
parser.add_argument(
    "--job_name",
    type=str,
)
parser.add_argument(
    "--list_of_questions",
    type=str,
    default="",
    help="question1+question2+...",
)
parser.add_argument(
    "--object_name",
    type=str,
)
parser.add_argument(
    "--object_name_1",
    type=str,
)
parser.add_argument(
    "--object_name_2",
    type=str,
)
parser.add_argument(
    "--output_object_name",
    type=str,
)
parser.add_argument(
    "--overwrite",
    type=int,
    default=0,
    help="0|1",
)
parser.add_argument(
    "--review_options",
    type=str,
    default="",
    help="dryrun,publish",
)
parser.add_argument(
    "--suffix",
    type=str,
    default="",
    help="<suffix>",
)
parser.add_argument(
    "--verbose",
    type=bool,
    default=0,
    help="0|1",
)
parser.add_argument(
    "--do_dryrun",
    type=int,
    default=0,
    help="0|1",
)
args = parser.parse_args()

success = False
if args.task == "combine":
    success = combine(
        args.object_name_1,
        args.object_name_2,
        args.object_name,
    )
elif args.task == "generate_multiple_review_workflow":
    success = generate_multiple_review_workflow(
        job_name=args.job_name,
        list_of_questions=args.list_of_questions.split("+"),
        object_name=args.object_name,
        review_options=args.review_options,
        do_publish=args.do_publish == 1,
        suffix=args.suffix,
        args=base64.b64decode(args.args).decode("utf-8").replace("\n", ""),
        do_dryrun=args.do_dryrun == 1,
    )
elif args.task == "review":
    success = review_literature(
        count=args.count,
        filename=args.filename,
        input_object_name=args.input_object_name,
        output_object_name=args.output_object_name,
        overwrite=args.overwrite == 1,
        question=args.question,
        verbose=args.verbose == 1,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from typing import List
from blueness import module
from notebooks_and_scripts.workflow.generic import Workflow
from openai_commands import NAME
from openai_commands.logger import logger

NAME = module.name(__file__, NAME)


def generate_workflow(
    job_name: str,
    object_name: str,
    list_of_questions: List[str],
    review_options: str,
    args: str,
    do_publish: bool = False,
    suffix: str = "",
    do_dryrun: bool = False,
) -> bool:
    if suffix == "-":
        suffix = ""

    logger.info(
        "{}.generate_workflow({}) {} -{}{}{}{}-> {}".format(
            NAME,
            job_name,
            object_name,
            review_options,
            "-{} question(s): {}".format(
                len(list_of_questions),
                ", ".join(list_of_questions),
            ),
            "-publish" if do_publish else "",
            args.replace("\n", " | "),
            f"suffix={suffix}" if suffix else "",
        )
    )

    workflow = Workflow(job_name)

    workflow.G.add_node("combination")

    for question in list_of_questions:
        workflow.G.add_node(question)
        workflow.G.nodes[question]["command_line"] = " ".join(
            [
                "workflow monitor",
                f"node={question}",
                workflow.job_name,
                "literature_review",
                "question={},suffix={},{}".format(
                    question,
                    suffix,
                    review_options,
                ),
                object_name,
                args,
            ]
        )

        workflow.G.add_edge("combination", question)

    output_object_name = "-".join([object_name, suffix] + list_of_questions)

    workflow.G.nodes["combination"]["command_line"] = " ".join(
        [
            "workflow monitor",
            f"node=combination,publish_as={output_object_name}",
            workflow.job_name,
            "literature_review",
            "combine",
            "dryrun={},publish={}".format(
                int(do_dryrun),
                int(do_publish),
            ),
        ]
        + ["-".join([object_name, suffix, question]) for question in list_of_questions]
        + [output_object_name]
    )

    return workflow.save()



from typing import List
from openai_commands.completion.prompts.structured import structured_prompt


class i2i_prompt(structured_prompt):
    def __init__(
        self,
        returns: str,
        *args,
        requirements: List[str] = None,
        **kwargs,
    ):
        super().__init__(
            inputs=["an image as a numpy array"],
            requirements=[
                "does not run a for loop on the pixels",
                "uses numpy vector functions",
                "imports all modules that are used in the code",
                "type-casts the output correctly",
            ]
            + ([] if requirements is None else requirements),
            returns=[f"{returns} as a numpy array"],
            *args,
            **kwargs,
        )



import os

from blue_objects import file, README

from openai_commands.literature_review.README import items as literature_review_items
from openai_commands import NAME, VERSION, ICON, REPO_NAME


default_thumbnail = (
    "https://github.com/kamangir/assets/raw/main/blue-plugin/marquee.png?raw=true"
)

features = {
    "literature review": {
        "description": "literature review using OpenAI API.",
        "icon": ICON,
        "thumbnail": "https://github.com/kamangir/assets/blob/main/openai_commands/literature-review/marquee.png?raw=true",
        "url": "https://github.com/kamangir/openai-commands/tree/main/openai_commands/literature_review",
    },
    "OpenAI Vision API": {
        "description": "a command interface to the [OpenAI vision API](https://platform.openai.com/docs/guides/vision).",
        "icon": ICON,
        "thumbnail": "https://raw.githubusercontent.com/kamangir/assets/main/vanwatch/2023-11-25-openai-vision/ButeNorthDavie.jpg",
        "url": "https://github.com/kamangir/openai-commands/tree/main/openai_commands/vision",
    },
    "code generation": {
        "description": "example notebooks to [generate python functions](./notebooks/completion_ai_function_py.ipynb), special case for [image to image python functions](./notebooks/completion_i2i_function.ipynb), and to [write a bash script](./notebooks/completion_ai_function_bash.ipynb) to use a script, for example, [vancouver-watching](https://github.com/kamangir/vancouver-watching).",
        "icon": ICON,
        "thumbnail": "https://github.com/kamangir/openai-commands/blob/main/assets/completion_i2i_function.png?raw=true",
        "url": "https://github.com/kamangir/openai-commands/tree/main/openai_commands/completion#%EF%B8%8F-code-generation",
    },
    "gpt": {
        "description": "co-authored with ChapGPT.",
        "icon": ICON,
        "thumbnail": default_thumbnail,
        "url": "https://github.com/kamangir/openai-commands/tree/main/openai_commands/gpt",
    },
    "VisuaLyze": {
        "description": 'How about calling it "VisuaLyze"? This name combines "visualize" and "analyze," reflecting the tool\'s capability to generate custom data visualizations and analyze user input through AI - OpenAI',
        "icon": ICON,
        "thumbnail": "https://github.com/kamangir/openai-commands/assets/1007567/7c0ed5f7-6941-451c-a17e-504c6adab23f",
        "url": "https://github.com/kamangir/openai-commands/tree/main/openai_commands/VisuaLyze",
    },
    "prompt completion": {
        "description": "basic prompt completion.",
        "icon": ICON,
        "thumbnail": default_thumbnail,
        "url": "https://github.com/kamangir/openai-commands/tree/main/openai_commands/completion#%EF%B8%8F-prompt-completion",
    },
    "image generation": {
        "description": "sentence -> image, text -> mural, images",
        "icon": ICON,
        "thumbnail": "https://github.com/kamangir/openai-commands/blob/main/assets/DALL-E.png?raw=true",
        "url": "https://github.com/kamangir/openai-commands/tree/main/openai_commands/images",
    },
    "template": {
        "description": "",
        "icon": ICON,
        "thumbnail": default_thumbnail,
        "url": "",
    },
}


items = [
    "{}[`{}`]({}) [![image]({})]({}) {}".format(
        details["icon"],
        feature,
        details["url"],
        details["thumbnail"],
        details["url"],
        details["description"],
    )
    for feature, details in features.items()
    if feature != "template"
]


def build():
    return README.build(
        items=items,
        path=os.path.join(file.path(__file__), ".."),
        NAME=NAME,
        VERSION=VERSION,
        REPO_NAME=REPO_NAME,
    ) and README.build(
        items=literature_review_items,
        cols=2,
        path=os.path.join(file.path(__file__), "literature_review"),
        NAME=NAME,
        VERSION=VERSION,
        REPO_NAME=REPO_NAME,
    )



import os
import matplotlib.pyplot as plt

from blue_options import string
from blue_options.host import is_jupyter
from blue_objects import file
from blue_objects.env import HOME

from openai_commands.completion.functions.python import ai_function_py
from openai_commands.logger import logger


class i2i_function(ai_function_py):
    def __init__(
        self,
        *args,
        plot=None,
        **kwargs,
    ):
        super().__init__(
            *args,
            output_class_name="ndarray",
            **kwargs,
        )

        self.auto_save = True
        self.plot = is_jupyter() if plot is None else plot

        if self.validation_input is None:
            _, self.validation_input = file.load_image(
                os.path.join(
                    HOME,
                    "git/blue-bracket/images/portal-34.jpg",
                )
            )

    def compute(self, inputs):
        if self.function_handle is None:
            return None

        logger.info(
            "{}.compute({})".format(
                self.__class__.__name__,
                string.pretty_shape_of_matrix(inputs),
            )
        )

        output_image = super().compute(inputs)

        logger.info(
            "->{}".format(
                string.pretty_shape_of_matrix(output_image),
            )
        )

        if self.plot:
            _, axes = plt.subplots(1, 2, figsize=(10, 5))
            axes[0].imshow(inputs)
            axes[0].axis("off")
            axes[0].set_title("input image")

            axes[1].imshow(output_image)
            axes[1].axis("off")
            axes[1].set_title("output image")
            plt.subplots_adjust(wspace=0.1)
            plt.show()

        return output_image



NAME = "openai_commands"

ICON = "🛠️"

DESCRIPTION = f"{ICON} a command interface to the OpenAI API."

VERSION = "3.178.1"

REPO_NAME = "openai-commands"

MARQUEE = "https://github.com/kamangir/openai-commands/raw/main/assets/carrot.png"



import argparse
from blueness import module
from blue_options.options import Options
from openai_commands import NAME, VERSION
from openai_commands.vision.completion import complete_object, Detail
from openai_commands.logger import logger
from blueness.argparse.generic import sys_exit

NAME = module.name(__file__, NAME)


parser = argparse.ArgumentParser(NAME, description=f"{NAME}-{VERSION}")
parser.add_argument(
    "task",
    type=str,
    help="complete",
)
parser.add_argument(
    "--detail",
    type=str,
    default="auto",
    help="auto|low|high",
)
parser.add_argument(
    "--object_name",
    type=str,
    default="",
)
parser.add_argument(
    "--options",
    type=str,
    default="",
    help="Davie,~Bute,.jpg",
)
parser.add_argument(
    "--max_count",
    type=int,
    default=5,
)
parser.add_argument(
    "--prompt",
    type=str,
    default="",
)
parser.add_argument(
    "--verbose",
    type=int,
    default=0,
    help="0|1",
)
args = parser.parse_args()

success = False
if args.task == "complete":
    success = complete_object(
        detail=Detail[args.detail.upper()],
        max_count=args.max_count,
        object_name=args.object_name,
        options=Options(args.options),
        prompt=args.prompt,
        verbose=args.verbose,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



import argparse
import os

from blueness import module
from blueness.argparse.generic import sys_exit
from blue_options.options import Options
from blue_objects import file, objects

from articraft import html
from openai_commands import NAME, VERSION
from openai_commands.DALLE.canvas import Canvas
from openai_commands.logger import logger

NAME = module.name(__file__, NAME)


parser = argparse.ArgumentParser(NAME, description=f"{NAME}-{VERSION}")
parser.add_argument(
    "task",
    type=str,
    help="render",
)
parser.add_argument(
    "--footer",
    type=int,
    default=0,
)
parser.add_argument(
    "--header",
    type=int,
    default=0,
)
parser.add_argument(
    "--source",
    type=str,
)
parser.add_argument(
    "--destination",
    type=str,
)
parser.add_argument(
    "--options",
    type=str,
    help="brush=tiling|randomwalk,brush_size={},~dryrun,lines=-1,url,verbose".format(
        os.getenv("DALL_E_BRUSH_SIZES", "")
    ),
)

args = parser.parse_args()

success = False
if args.task == "render":
    options = Options(args.options)
    brush_kind = options.get("brush", "tiling")
    brush_size = options.get("brush_size", 256)
    dryrun = options.get("dryrun", 1) == 1
    is_url = options.get("url", 0) == 1
    lines = options.get("lines", -1)
    verbose = options.get("verbose", 0) == 1

    if is_url:
        success, content = html.ingest_poetry_from_url(
            args.source,
            args.header,
            args.footer,
        )
    else:
        success, content = file.load_text(args.source)

    output_filename = args.destination
    if not output_filename:
        if not is_url:
            output_filename = file.add_extension(args.source, "png")
        else:
            output_filename = objects.path_of("DALL-E.png")

    if success:
        content = content[:lines] if lines != -1 else content

        logger.info(
            f"DALL-E: render: {args.source} -{len(content)} lines-> {output_filename}"
        )

        canvas = Canvas(
            brush_kind=brush_kind,
            brush_size=brush_size,
            content=content,
            dryrun=dryrun,
            source=args.source,
            verbose=verbose,
        )

        canvas.render_text(
            canvas.create_brush(
                brush_kind,
                brush_size,
            ),
            content,
            output_filename,
        )

else:
    success = None

sys_exit(logger, NAME, args.task, success)



import argparse
from blueness import module
from tqdm import tqdm

from blueness.argparse.generic import sys_exit
from blue_options import string
from blue_objects import file, objects

from openai_commands import NAME, VERSION
from openai_commands.images.api import OpenAIImageGenerator
from openai_commands.logger import logger

NAME = module.name(__file__, NAME)


parser = argparse.ArgumentParser(NAME, description=f"{NAME}-{VERSION}")
parser.add_argument(
    "task",
    type=str,
    help="generate",
)
parser.add_argument(
    "--object_name",
    type=str,
    default=".",
)
parser.add_argument(
    "--options",
    type=str,
    default="",
)
parser.add_argument(
    "--filename",
    type=str,
    default=f"{string.timestamp()}.png",
)
parser.add_argument(
    "--prompt",
    type=str,
    default="",
    help="prompt+prompt+prompt",
)
parser.add_argument(
    "--verbose",
    type=int,
    default=0,
    help="0|1",
)
args = parser.parse_args()

success = False
if args.task == "generate":
    generator = OpenAIImageGenerator(verbose=args.verbose)

    success = True
    for index, prompt in tqdm(enumerate(args.prompt.split("+"))):
        filename = objects.path_of(
            filename=file.add_suffix(args.filename, f"{index:05d}"),
            object_name=args.object_name,
            create=True,
        )

        if not generator.generate(
            prompt=prompt,
            filename=filename,
        )[0]:
            success = False
else:
    success = None

sys_exit(logger, NAME, args.task, success)
